<?php

class Pixtacy_Products {

	private static $allImagesBuyable = FALSE;
	private static $noImageBuyable = FALSE;
	private static $globalProducts; // Gesamtliste der theoretisch kaufbaren Produkte
	private static $autoProducts; // Liste der automatisch zugeordneten theoretisch kaufbaren Produkte
	private static $assortments = array (); // Gesamtliste der Sortimente mit ihren Produkten (Sortiment-ID = Schlüssel)
	private static $assortmentsAlwaysBuyable = array (); // Liste der Sortimente, die stets kaufbare Produkte enthalten
	private static $productAssortments; // Liste mit Produkt-IDs (= Schlüssel) und Sortimenten, zu denen das Produkt gehört
	private static $options; // Datensatz mit globalen Optionen
	private static $productQuantities; // Liste der Warenkorbprodukte mit ihren (aufsummierten) Mengen
	private static $imageQuantities; // Liste der Warenkorbbilder mit ihren Mengen je Produkt
	public static $cart = [];
	public static $lastProductsCount;


	public function __set( $name, $value ){
		self::$options[ $name ] = $value;
	}


	public static function set_options( $options ){
		self::$options = $options;
	}

	// initialize() has to be called prior to other methods
	public static function initialize( $db, $pageIdProducts, $pageIdAssortments, $cart = NULL ){
		if ( empty( $db ) || empty( $pageIdProducts ) || empty( $pageIdAssortments ) ){
			return FALSE;
		}
		$pageIdProducts = (int) $pageIdProducts;
		$pageIdAssortments = (int) $pageIdAssortments;
		if ( $cart ){
			self::$cart = is_array( $cart ) ? $cart : json_decode( $cart, TRUE );
		}

		// build product assortment mapping table
		$sql = "SELECT c.`id`, a.`liste` 
			FROM `%data_assortment` a LEFT JOIN `%content` c ON c.id = a.vtid 
			WHERE c.containerID = {$pageIdAssortments}";
		$result = $db->query( $sql );
		while ( $result && $assortment = $result->fetch_assoc() ) {
			$elements = json_decode( $assortment['liste'], TRUE );
			self::$assortments[ $assortment['id'] ] = $elements;
			foreach ( $elements as $element ) {
				if ( ! empty( $element['product_id'] ) ) {
					self::$productAssortments[ $element['product_id'] ][] = $assortment['id'];
				}
			}
		}

		// build lists of buyable products and assortments
		$groups = empty( self::$options['userGroups'] ) ? array () : explode( ',', self::$options['userGroups'] );
		$sql = "SELECT c.id, p.`benutzer`, p.`kennung`, p.`preis:num`, p.`zuordnung:num`, p.`typ`,
			p.`bezeichnung`, p.`bezeichnung_en`, p.`beschreibung`, p.`beschreibung_en`, p.`preisanpassung`, p.`steuersatz`
			FROM `%data_product` p LEFT JOIN `%content` c ON c.id = p.vtid 
			WHERE c.containerID = {$pageIdProducts}
			ORDER BY IF( c.sortValue > 0, 0, 1 ), c.sortValue, p.bezeichnung";
		$result = $db->query( $sql );
		//if ( ! $result ){
		//	die( $db->error );
		//}

		while ( $result && $result->num_rows && $product = $result->fetch_assoc() ) {
			if ( ( empty( self::$options['paymentsPossible'] ) && $product['preis:num'] )
				|| ( empty( self::$options['deliveryPossible'] ) && strtolower( $product['typ'] ) == 'print' )
				|| ( ! empty( $product['benutzer'] ) && ! in_array( $product['benutzer'], $groups ) 
				&& ! ( $product['benutzer'] == 'guest' && empty( self::$options['userName'] ) )
				&& ! ( $product['benutzer'] == 'Alle' && ! empty( self::$options['userName'] ) ) ) 
			) {
				continue; // Produkt ist generell nicht kaufbar
			}
			$prodId = (int) $product['id'];
			self::$globalProducts[ $prodId ] = $product;
			if ( $product['zuordnung:num'] ) {
				// es ist ein automatisch zugeordnetes Produkt
				self::$autoProducts[] = $prodId;
				if ( empty( $product['kennung'] ) ) {
					// es ist nicht von High-res-Datei abhängig
					self::$allImagesBuyable = TRUE;
				}
			}
			elseif ( ! isset( self::$productAssortments[ $prodId ] ) ) { 
				// Produkt ist keinem Sortiment zugeordnet
				continue;
			}
			elseif ( empty( $product['kennung'] ) ) { 
				// Produkt ist nicht von High-res-Datei abhängig
				// setze alle Sortimente, denen das Produkt zugeordnet ist, auf "generell kaufbar"
				foreach ( self::$productAssortments[ $prodId ] as $assortmentId ) {
					self::$assortmentsAlwaysBuyable[] = $assortmentId;
				}
			}
		}
		return array ( 'assortments' => self::$assortments, 'assortmentsAlwaysBuyable' => self::$assortmentsAlwaysBuyable,
			'globalProducts' => self::$globalProducts, 'autoProducts' => self::$autoProducts );
	}

	public static function get_offerings( $imageAssortment, $imageProducts, $galleryAssortment, $galleryProducts ){
		$assortment = $imageAssortment == -1 ? NULL : (int) $imageAssortment;
		$products = $imageProducts ? explode( "\n", $imageProducts ) : array ();
		// Automatisch zugeordnete Produkte
		if ( ! empty( self::$autoProducts ) ){
			$products = array_merge( $products, self::$autoProducts );
		}
		// Galeriesortiment
		if ( empty( $imageAssortment ) ){
			if ( $galleryAssortment == 0 ){
				// Galerie hat kein Sortiment, sondern eigene Produktauswahl
				$products = array_merge( $products, explode( "\n", $galleryProducts ) );
			}
			else {
				$assortment = $galleryAssortment == -1 ? NULL : (int) $galleryAssortment;
			}
		}
		return compact( 'assortment', 'products' );
	}

	public static function is_image_buyable( $imagePath, $imageAssortment, $imageProducts, 
		$galleryAssortment, $galleryProducts )
	{
		if ( self::$allImagesBuyable ) {
			return TRUE;
		}
		if ( self::$noImageBuyable ) {
			return FALSE;
		}
		$offerings = self::get_offerings( $imageAssortment, $imageProducts, $galleryAssortment, $galleryProducts );
		$products = array ();
		if ( isset( self::$assortments[ $offerings['assortment'] ] ) ){
			if ( in_array( $offerings['assortment'], self::$assortmentsAlwaysBuyable ) ){
				return TRUE;
			}
			foreach ( self::$assortments[ $offerings['assortment'] ] as $item ){
				if ( ! empty( $item['product_id'] ) ){
					$products[] = $item['product_id'];
				}
			}
		}
		$products = array_merge( $products, $offerings['products'] );
		foreach ( $products as $product ) {
			if ( isset( self::$globalProducts[ $product ] ) 
				&& ( self::$globalProducts[ $product ]['kennung'] == '' 
				|| hires_file_exists( $imagePath, self::$globalProducts[ $product ]['kennung'] ) )
			) {
				return TRUE;
			}
		}
		return FALSE;
	}


	public static function get_products( $imageId, $imagePath, $imageAssortment, $imageProducts, $galleryAssortment, $galleryProducts ){
		$products = array ();
		$productIds = array ();
		$offerings = self::get_offerings( $imageAssortment, $imageProducts, $galleryAssortment, $galleryProducts );
		// prüfe Produkte des zugeordneten Sortiments
		if ( isset( self::$assortments[ $offerings['assortment'] ] ) ) {
			foreach ( self::$assortments[ $offerings['assortment'] ] as $item ){
				if ( empty( $item['product_id'] ) && ! empty( $item['text'] ) ){
					$products[] = array ( 'id' => 0, 'name' => $item['text'], 'name_en' => $item['text_en'] );
				}
				elseif ( empty( self::$globalProducts[ $item['product_id'] ]['kennung'] ) 
					|| hires_file_exists( $imagePath, self::$globalProducts[ $item['product_id'] ]['kennung'] ) 
				){
					$products[] = self::get_product_record( $item['product_id'], $imageId );
					$productIds[] = $item['product_id'];
				}
			}
		}
		// prüfe individuell zugeordnete Produkte
		foreach ( $offerings['products'] as $productId ) {
			if ( isset( self::$globalProducts[ $productId ] ) 
				&& ! in_array( $productId, $productIds )
				&& ( empty( self::$globalProducts[ $productId ]['kennung'] )
				|| hires_file_exists( $imagePath, self::$globalProducts[ $productId ]['kennung'] ) )
			) {
				$products[] = self::get_product_record( $productId, $imageId );
				$productIds[] = $productId;
			}
		}
		// liefere Produktliste zurück
		self::$lastProductsCount = count( $products );
		return $products;
	}


	public static function get_cart_price( $prodId ){
		if ( empty( self::$globalProducts[ $prodId ] ) ){
			return FALSE;
		}
		if ( empty( self::$cart ) ){
			return 0;
		}
		if ( is_null( self::$productQuantities ) ){
			self::$productQuantities = array ();
			foreach ( self::$cart as $item ){
				self::$productQuantities[ $item['product'] ] = 
				isset( self::$productQuantities[ $item['product'] ] ) 
					? self::$productQuantities[ $item['product'] ] + $item['quantity'] 
				: $item['quantity'];
			}
		}
		$quantity = isset( self::$productQuantities[ $prodId ] ) 
			? self::$productQuantities[ $prodId ] 
			: 1;
		$product = self::$globalProducts[ $prodId ];
		$price = (float) $product['price'];
		$factor = empty( self::$options['userPriceFactor'] ) 
			? 1 
			: (float) str_replace( ',', '.', self::$options['userPriceFactor'] );
		if ( ! empty( $product['applyDiscount'] ) && ! empty( $product['discountScale'] ) ){
			foreach ( $product['discountScale'] as $level ){
				if ( $quantity >= $level['quantity'] ){
					$price = (float) $level['price'];
				}
			}
		}
		if ( $product['applyUserPriceFactor'] && $factor != 1 ){
			$price = $price * $factor;
		}
		if ( empty( self::$options['vatHandling'] ) || self::$options['vatHandling'] == 'excluded' ){
			return $price;
		}
		$vatrate = self::$options[ $product['vatRate1'] == 2 ? 'vatRate2' : 'vatRate1' ];
		return round( $price / ( 1 + $vatrate / 100 ), 2 );
	}


	public static function get_cart_quantity( $imageId, $prodId ){
		if ( ! isset( self::$imageQuantities ) && ! empty( self::$cart ) ){
			self::$imageQuantities = array ();
			foreach ( self::$cart as $item ){
				self::$imageQuantities[ $item['image'] ][ $item['product'] ] = $item['quantity'];
			}
		}
		if ( isset( self::$imageQuantities[ $imageId ][ $prodId ] ) ){
			return self::$imageQuantities[ $imageId ][ $prodId ];
		}
		return 0;
	}


	private static function get_product_record( $prodId, $imageId ){
		if ( ! isset( self::$globalProducts[ $prodId ] ) ){
			return FALSE;
		}
		$p = self::$globalProducts[ $prodId ];
		if ( ! empty( self::$options['preferEnglish'] ) && ! empty( $p['bezeichnung_en'] ) ){
			$name = $p['bezeichnung_en'];
		}
		else {
			$name = $p['bezeichnung'];
		}
		return array (
			'id' => (int) $prodId,
			'name' => trim( strtok( $name, '@' ) ),
			'type' => $p['typ'],
			'description' => ! empty( self::$options['preferEnglish'] ) && ! empty( $p['beschreibung_en'] ) ? $p['beschreibung_en'] : $p['beschreibung'],
			'price' => $p['preis:num'],
			'applyUserPriceFactor' => ! empty( $p['preisanpassung'] ),
			'vatRate' => self::$options[ $p['steuersatz'] == 2 ? 'vatRate2' : 'vatRate1' ],
			'applyDiscount' => ! empty( $p['rabatt_anwenden'] ),
			'discountScale' => empty( $p['rabattstaffel'] ) 
				? NULL 
				: json_decode( $p['rabattstaffel'] ),
			'cartVatRate' => self::$options[ $p['steuersatz'] == 2 ? 'vatrate2' : 'vatrate1' ],
			'cartQuantity' => self::get_cart_quantity( $imageId, $prodId ),
		);
	}
}

// end of file pxtc_products.class.php

