/*! lg-flagging - v0.9 - 2023-01-14
* Copyright (c) 2023 Martin Wandelt */

(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.lgFlagging = factory());
}( this, ( function () { 'use strict';

	var __assign = function() {
		__assign = Object.assign || function __assign(t) {
			for (var s, i = 1, n = arguments.length; i < n; i++) {
				s = arguments[i];
				for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
			}
			return t;
		};
		return __assign.apply(this, arguments);
	};

	var flaggingSettings = {
		flagging: true,
		hideUnflaggedElements: false,
	};

	/**
	* List of lightGallery events
	* All events should be documented here
	* Below interfaces are used to build the website documentations
	* */
	var lGEvents = {
		afterAppendSlide: 'lgAfterAppendSlide',
		init: 'lgInit',
		hasVideo: 'lgHasVideo',
		containerResize: 'lgContainerResize',
		updateSlides: 'lgUpdateSlides',
		afterAppendSubHtml: 'lgAfterAppendSubHtml',
		beforeOpen: 'lgBeforeOpen',
		afterOpen: 'lgAfterOpen',
		slideItemLoad: 'lgSlideItemLoad',
		beforeSlide: 'lgBeforeSlide',
		afterSlide: 'lgAfterSlide',
		posterClick: 'lgPosterClick',
		dragStart: 'lgDragStart',
		dragMove: 'lgDragMove',
		dragEnd: 'lgDragEnd',
		beforeNextSlide: 'lgBeforeNextSlide',
		beforePrevSlide: 'lgBeforePrevSlide',
		beforeClose: 'lgBeforeClose',
		afterClose: 'lgAfterClose',
		rotateLeft: 'lgRotateLeft',
		rotateRight: 'lgRotateRight',
		flipHorizontal: 'lgFlipHorizontal',
		flipVertical: 'lgFlipVertical',
		autoplay: 'lgAutoplay',
		autoplayStart: 'lgAutoplayStart',
		autoplayStop: 'lgAutoplayStop',
	};

	var Flagging = ( function( element ){

		function Flagging( instance, $LG ) {
			// get lightGallery core plugin instance
			this.core = instance;
			this.$LG = $LG;
			this.settings = __assign(__assign({}, flaggingSettings), this.core.settings);
			return this;
		}

		Flagging.prototype.init = function() {

			if ( ! this.settings.flagging ) {
				return;
			}

			var sheet = document.createElement('style')
			sheet.innerHTML = 
				'.lg-flagging-box { position:absolute; top:5px; left:9px; cursor:pointer; z-index:9999 } ' +
				'.lg-flagging-box i { color:white; opacity:0.6; text-shadow: 0 0 1em #888, 0 0 0.2em #888; font-size: 1.75rem } ' +
				'.lg-flagging-box.flagged i { font-weight:900; color:yellow; opacity:0.9; text-shadow: 1px 1px 2px black, 0 0 1em grey, 0 0 0.2em grey; } ' +
				'.lg-flagging-box:hover i { opacity:1; text-shadow: 1px 1px 2px black, 0 0 1em grey, 0 0 0.2em grey; } ' +
				'.lg-flagging-box.flagged:hover i { text-shadow: 1px 1px 2px black, 0 0 1em black, 0 0 0.2em black; }';
			document.head.appendChild( sheet );

			var _this = this;
			var html = '<div class="lg-flagging-box position-absolute p-4"><i class="far fa-heart"></i></div>';
			this.core.outer.find('.lg-content').append( html );
			this.flaggingbox = this.core.outer.find('.lg-flagging-box');
			window.setTimeout( function(){ _this.adjustBoxPosition(); }, 1000 );

			$(window).on( 'resize', function(){
				_this.adjustBoxPosition();
			});

			this.core.LGel.on( lGEvents.afterSlide, function( event, prevIndex, index ){
				_this.core.currentIndex = index;
				_this.updateFlag();
			});

			this.core.LGel.on( lGEvents.beforeSlide, function( event, prevIndex, index ){
				_this.flaggingbox.css( 'display', 'none' );
				_this.flaggingbox.removeClass('flagged');
			});

			this.core.LGel.on( lGEvents.slideItemLoad, function(){
				_this.updateFlag();
			});

			this.core.LGel.on( lGEvents.posterClick, function(){
				_this.updateFlag();
			});

			this.core.outer.find('.lg-flagging-box').first().on( 'click', function(){
				var url = $(this).hasClass('flagged') ? urlLightboxRemove : urlLightboxAdd;
				var elm = $( '#' + _this.getAttribute('elementId') );
				var id = elm.data('id');
				$.get( url + '&amp;vtid=' + id, function( data ){
					if ( data.indexOf( '|' ) != -1 ){
						var html = $.parseHTML( data );
						if ( $(html).find('form[name=anmeldung]').length ){
							window.location.reload();
							return;
						}
						var parts = data.split('|');
						parts[3] > 0 ? $('.lbcount').text( parts[3] ).parents('li').show() : $('.lbcount').parents('li').hide();
						$('.lg-flagging-box').toggleClass('flagged');
						elm.toggleClass('flagged');
						if ( _this.settings.hideUnflaggedElements ){
							elm.hasClass('flagged') ? elm.show() : elm.hide();
						}
					}
				});
			});
		};

		Flagging.prototype.updateFlag = function(){
			var elm = $( '#' + this.getAttribute('elementId') );
			if ( elm.hasClass('flagged') ){
				this.flaggingbox.addClass('flagged');
			}
			this.adjustBoxPosition();
			this.flaggingbox.css( 'display', 'block' );
		}

		Flagging.prototype.adjustBoxPosition = function(){
			var outerPos = this.core.outer.offset();
			var pos = this.core.outer.find('.lg-current .lg-image').offset();
			var toolbarHeight = this.core.$toolbar.firstElement.offsetHeight;
			if ( pos ){
				this.flaggingbox.css( 'top', '' + ( pos.top - outerPos.top - toolbarHeight ) + 'px' );
				this.flaggingbox.css( 'left', '' + pos.left + 'px' );
			}
		}

		Flagging.prototype.getAttribute = function( prop ){
			var index = this.core.index;
			return this.core.galleryItems[ index ][ prop ];
			//return this.core.$items.eq( index ).data( prop );
		}

		Flagging.prototype.destroy = function() {
		};

		return Flagging;
	}());

	return Flagging;
})));

